"""Python setup.py for developergpt package"""
import io
import os
from setuptools import find_packages, setup


def read(*paths, **kwargs):
    """Read the contents of a text file safely.
    >>> read("developergpt", "VERSION")
    '0.1.0'
    >>> read("README.md")
    ...
    """

    content = ""
    with io.open(
        os.path.join(os.path.dirname(__file__), *paths),
        encoding=kwargs.get("encoding", "utf8"),
    ) as open_file:
        content = open_file.read().strip()
    return content


def read_requirements(path):
    return [
        line.strip()
        for line in read(path).split("\n")
        if not line.startswith(('"', "#", "-", "git+"))
    ]


setup(
    name="developergpt",
    version=read("developergpt", "VERSION"),
    python_requires=">=3.9",
    description="DeveloperGPT is a LLM-powered command line tool that enables natural language to terminal commands and in-terminal chat.",
    url="https://github.com/luo-anthony/DeveloperGPT/",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="luo-anthony",
    packages=find_packages(exclude=["tests", ".github"]),
    install_requires=read_requirements("requirements.txt"),
    entry_points={"console_scripts": ["developergpt = developergpt.__main__:main"]},
    extras_require={"test": read_requirements("requirements-test.txt")},
)
